#!/usr/bin/python
#
# LBX archive module for Master Of Orion 2 Graphics Viewer
# Copyright (c) Karol "Wolverine" Tomala 2007
#
# This file is licensed under GPLv2
#

import sys,os
from struct import pack,unpack

# =========== LBX CLASS ============

class LBX:

    # --------- CONSTRUCTOR ----------

    def __init__(self):
	"""Class constructor"""
	# Magic number ADFE0000 (converted to little endian integer)
	self.LBXMagic=65197
    
    # ---------- INTERNAL FUNCTIONS ----------
    
    def __getheader(self,fp):
	"""getheader(fp): Retrieves file header"""
	self.numfiles,=unpack('<H',fp.read(2))
	self.magic,=unpack('<I',fp.read(4))
	if (self.magic!=self.LBXMagic):
	    print "Error: File "+self.filename+" isn't LBX archive!"
	    return False
	self.version,=unpack('<H',fp.read(2))
	self.offsets=[]
	for i in range(0,self.numfiles+1):
	    offset,=unpack('<I',fp.read(4))
	    self.offsets.append(offset)

    def __getfilepos(self):
	"""getfilepos(): Calculates file position offsets"""
	self.filepos=[]
	for i in range(0,len(self.offsets)-1):
	    start=self.offsets[i]
	    end=self.offsets[i+1]
	    size=end-start
	    self.filepos.append((start,size))

    def __extract(self,fp):
	"""extract(fp): Extracts data"""
	for i in range(0,len(self.filepos)):
	    offset,size=self.filepos[i]
	    fp.seek(offset)
	    fnum=str(i)
	    if (len(fnum)<4):
		fnum='0'*(4-len(fnum))+fnum
	    extfilename=self.filename+"_"+fnum
	    fpout=open(extfilename,'wb')
	    fpout.write(fp.read(size))
	    fpout.close()
	print str(len(self.filepos))+" files successfully extracted from "+self.filename

    # --------- METHODS ---------

    def load(self,filename):
	"""load(filename): Load filename"""
	self.filename=filename
	fsize=os.path.getsize(filename)
	fp=open(filename,'rb')
	if (self.__getheader(fp)==False):
	    return False
	self.__getfilepos()
	self.__extract(fp)
	fp.close()

    def create(self,filename,dirpath):
	"""create(filename,dirpath): Creates archive from dirpath with name filename"""
	# Creates (almost - except junk) identical LBX files as original ones
	files=[]
	for file in os.listdir(dirpath):
	    if os.path.isfile(dirpath+'/'+file):
		if (dirpath+'/'+file!=filename):
		    files.append((file,os.path.getsize(dirpath+'/'+file)))
	files.sort()
	filecount=len(files)
	if (filecount>509):
	    print "Error: Too many files to pack (max. 509)!"
	    return False
	
	# Starting offset (always the same)
	soffset=2048L
	
	# Calculate offsets
	offset=soffset
	offsets=[offset]
	for i in range(0,filecount):
	    offset+=files[i][1]
	    offsets.append(offset)
	
	# Zero fill count: startoffset - len(offsets)*4 - header length
	zeroes=soffset-len(offsets)*4-8
	
	# Open file
	fp=open(filename,'wb')
	# Create header
	fp.write(pack('<H',filecount))
	fp.write(pack('<I',self.LBXMagic))
	fp.write(pack('<H',0))
	# Create offsets
	for i in range(0,len(offsets)):
	    fp.write(pack('<I',offsets[i]))
	# Fill in zeroes
	for i in range(0,zeroes):
	    fp.write(pack('<B',0))
	# Add files
	for i in range(0,len(files)):
	    fpin=open(dirpath+'/'+files[i][0],'rb')
	    fp.write(fpin.read(files[i][1]))
	    fpin.close()
	# Close file
	fp.close()
	
	    

# Break if called not as module
if __name__=='__main__':
    sys.stderr.write('Error: This is module. Run main.py instead!')
    sys.stderr.flush()
    sys.exit(1)
