#
# PaletteBox Widget for PyQT
# Copyright (c) Karol "Wolverine" Tomala 2007
# Contact: karol at karoltomala dot com
#
# Licensed under LGPLv2 (Library General Public License Version 2)
# See Free Software Foundation website (www.fsf.org) for details
#

from qt import *
import types

# ============== PALETTEBOX WIDGET ===============

class PaletteBox(QWidget):

    # -------- CONSTRUCTOR --------

    def __init__(self,*args):
	apply(QWidget.__init__,(self,)+args)
	self.__pal=None
	self.__hover=False
	self.__palw=10
	self.__palh=10
	self.__curhover=None
	self.__selected=None
	self.__paint=QPainter()
	self.__allowsel=True
	self.__showsel=True
	self.setMouseTracking(0)

    # -------- INTERNAL --------
    
    # Draws palette
    def __draw(self):
	if (self.__pal==None):
	    return
    
	self.__paint.begin(self)

	x=0
	y=0
	w=self.__palw
	h=self.__palh
	
	# Paint palette
	linecols=self.size().width()/self.__palw
	colors=len(self.__pal)
	lines=colors/linecols
	if (lines*linecols<colors):
	    lines+=1
	for ln in range(0,lines):
	    y=self.__palh*ln
	    for col in range(0,linecols):
		if (ln*linecols+col>=colors):
		    break
		x=self.__palw*col
		self.__drawRectangle(x,y,w+1,h+1,"#000000")
		self.__drawRectangle(x+1,y+1,w-1,h-1,self.__pal[ln*linecols+col],self.__pal[ln*linecols+col])

	self.__paint.flush()
	self.__paint.end()

    # Draw highlight of color selcol
    def __drawhighlight(self,selcol,color="#ffffff",border=None):
	if (self.__pal==None):
	    return
	w=self.__palw
	h=self.__palh
	linecols=self.size().width()/self.__palw

	# Paint selection
	if (selcol!=None):
	    selline=selcol/linecols
	    selrelx=selcol-selline*linecols
	    self.__paint.begin(self)
	    if (border!=None):
	        self.__drawRectangle(selrelx*self.__palw+2,selline*self.__palh+2,w-2,h-2,color,border=border)
	    else:
	        self.__drawRectangle(selrelx*self.__palw,selline*self.__palh,w+1,h+1,color)
	    self.__paint.flush()
	    self.__paint.end()

    # Draws rectangle
    def __drawRectangle(self,x,y,w,h,color,bgcolor=None,border=None):
	if (self.__pal==None):
	    return

	rect=QRect(x,y,w,h)
	
	fg=QColor()
	fg.setNamedColor(color)
	
	pen=QPen()
	pen.setColor(fg)
	pen.setStyle(Qt.SolidLine)
	if (border!=None):
	     pen.setWidth(border)
	self.__paint.setPen(pen)
	
	if (bgcolor!=None):
	    bg=QColor()
	    bg.setNamedColor(bgcolor)
    	    b=QBrush()
	    b.setColor(bg)
	    b.setStyle(Qt.BrushStyle(Qt.SolidPattern))
	    self.__paint.setBrush(b)
	else:
    	    b=QBrush()
	    b.setStyle(Qt.BrushStyle(Qt.NoBrush))
	    self.__paint.setBrush(b)
	    
	self.__paint.drawRect(rect)
	
    # Gets color residing under position
    def __getColorFromPos(self,x,y):
	if (self.__pal==None):
	    return
	linecols=self.size().width()/self.__palw
	lines=len(self.__pal)/linecols+1
	if (x<1 or x>linecols*self.__palw-1):
	    return
	if (y<1 or y>lines*self.__palh-1):
	    return
	xr=x/self.__palw
	yr=y/self.__palh
	cursel=xr+linecols*yr
	if (cursel<0 or cursel>len(self.__pal)-1):
	    return
	return cursel

    # Draws border on selected color
    def __drawselected(self):
	if (self.__selected==None):
	    return
	self.__drawhighlight(self.__selected,"#dd44ff",2)
	self.__drawhighlight(self.__selected,"#000000")

    # -------- EVENTS --------

    # Handles repaint event
    def paintEvent(self,ev):
	self.__draw()
	self.__drawselected()

    # Handle mouse movement
    def mouseMoveEvent(self,ev):
	if (self.__pal==None):
	    return
	if (self.__hover==True):
	    x=ev.pos().x()
	    y=ev.pos().y()
	    lasthover=self.__curhover
	    curhover=self.__getColorFromPos(x,y)
	    if (curhover!=lasthover):
		if (lasthover==self.getSelected()):
		    self.__drawselected()
		else:
		    self.__drawhighlight(lasthover,"#000000")
		self.__drawhighlight(curhover)
	        self.__curhover=curhover

    # Handles mouse click
    def mousePressEvent(self,ev):
	if (self.getAllowSelect()==True):
	    x=ev.pos().x()
	    y=ev.pos().y()
	    cursel=self.__getColorFromPos(x,y)
	    if (cursel!=None and self.__showsel==True):
		self.setSelected(cursel)
		self.__draw()
	        self.__drawselected()

    # -------- METHODS --------

    # Sets palette
    def setPalette(self,palette):
	if (type(palette)!=types.ListType):
	    return
	self.__pal=palette
    
    # Returns palette
    def getPalette(self):
	return self.__pal

    # Sets hover highlight
    def setHover(self,hover=True):
	if (hover==True):
	    self.setMouseTracking(1)
	elif (hover==False):
	    self.setMouseTracking(0)
	else:
	    return
	self.__hover=hover
    
    # Returns hover status
    def getHover(self):
	return self.__hover

    # Sets selected color
    def setSelected(self,selected=None):
	if (selected<0 or selected>len(self.__pal)):
	    return
	self.__selected=selected

    # Returns selected color
    def getSelected(self):
	return self.__selected

    # Sets colorbox size
    def setColorSize(self,w,h):
	if (w<2 or w>self.size().width()):
	    return
	if (h<2 or h>self.size().height()):
	    return
	self.__palw=w
	self.__palh=h

    # Returns colorbox size
    def getColorSize(self):
	return (self.__palw,self.__palh)

    # Set selection allowance
    def setAllowSelect(self,allow=True):
	if (type(allow)!=types.BooleanType):
	    return
	self.__allowsel=allow

    # Returns selection allowance
    def getAllowSelect(self):
	return self.__allowsel

    # Sets selection showing
    def setShowSelect(self,showsel=True):
	if (type(showsel)!=types.BooleanType):
	    return
	self.__showsel=showsel	
    
    # Returns selection showing
    def getShowSelect(self):
	return self.__showsel

