#
# PaletteBox Widget for PyQT
# Copyright (c) Karol "Wolverine" Tomala 2007
# Contact: karol at karoltomala dot com
#
# Licensed under LGPLv2 (Library General Public License Version 2)
# See Free Software Foundation website (www.fsf.org) for details
#

from qt import *

# ============== VIEWBOX WIDGET ===============

class ViewBox(QWidget):

    # -------- CONSTRUCTOR --------

    def __init__(self,*args):
	apply(QWidget.__init__,(self,)+args)
	self.__paint=QPainter()
	self.setMouseTracking(0)
	self.__graphic=False
	self.__palette=False
	self.__curframe=False
	self.__framebuf=False
	self.__relposx=0
	self.__relposy=0
    
    # -------- INTERNAL --------
    
    # Draws rectangle
    def __drawRectangle(self,x,y,w,h,color,bgcolor=None,border=None):

	rect=QRect(x,y,w,h)
	
	fg=QColor()
	fg.setNamedColor(color)
	
	pen=QPen()
	pen.setColor(fg)
	pen.setStyle(Qt.SolidLine)
	if (border!=None):
	     pen.setWidth(border)
	self.__paint.setPen(pen)
	
	if (bgcolor!=None):
	    bg=QColor()
	    bg.setNamedColor(bgcolor)
    	    b=QBrush()
	    b.setColor(bg)
	    b.setStyle(Qt.BrushStyle(Qt.SolidPattern))
	    self.__paint.setBrush(b)
	else:
    	    b=QBrush()
	    b.setStyle(Qt.BrushStyle(Qt.NoBrush))
	    self.__paint.setBrush(b)
	    
	self.__paint.drawRect(rect)

    # Draws pattern rectangle
    def __drawGridPat(self):
	self.__paint.begin(self)
	
	rect=QRect(0,0,self.size().width(),self.size().height())
	
	img=QImage()
	img.load("res/grid.png")
	
	for x in range(0,self.size().width(),20):
	    for y in range(0,self.size().height(),20):
		self.__paint.drawImage(x,y,img)

	self.__paint.flush()
	self.__paint.end()

    # Creates frame buffer image
    def __createFrameBuf(self):
	frame=self.getFrame(self.getCurrentFrameNum())
	w,h=self.getGraphic().getwh()
	cx=cy=0
	frmindent=frame.getindent()
	relx=cx
	rely=cy+frmindent
	
	image=QImage(w,h,32)
	image.fill(0)
	
	for i in range(0,frame.countlines()):
	    line=frame.getline(i)
	    if (line.special==0):
		xindent=line.indent
		relx+=xindent
		for j in range(0,len(line.pixels)):
		    # FIXME: This is weird especially in planets.lbx
		    if (line.pixels[j]>=self.getPalette().countcolors()):
			col=self.getPalette().getcolor(0)
		    else:
			col=self.getPalette().getcolor(line.pixels[j])
		    c=QColor()
		    c.setNamedColor(col)
		    image.setPixel(relx+j,rely,c.rgb())
		relx+=len(line.pixels)
	    else:
		yindent=line.indent
		rely+=yindent
		relx=cx
	self.setFrameBuf(image)

    # Draws image
    def __drawImage(self):
	if (self.getFrameBuf()!=False):
	    w,h=self.getGraphic().getwh()
	    rx,ry=self.getRelPos()
	    if (self.size().width()<=w):
		cx=rx
	    else:
		cx=int((self.size().width()-w)/2)
	    if (self.size().height()<=h):
		cy=ry
	    else:
		cy=int((self.size().height()-h)/2)
	    
	    self.__paint.begin(self)
	    self.__paint.drawImage(cx,cy,self.getFrameBuf())
	    self.__paint.flush()
	    self.__paint.end()
	
	
    # -------- EVENTS --------

    # Handles repaint event
    def paintEvent(self,ev):
	self.__drawGridPat()
	self.__drawImage()

    # Handle mouse movement
#    def mouseMoveEvent(self,ev):
#	if (self.__hover==True):
#	    x=ev.pos().x()
#	    y=ev.pos().y()

    # Handles mouse click
#    def mousePressEvent(self,ev):
#	if (self.getAllowSelect()==True):
#	    x=ev.pos().x()
#	    y=ev.pos().y()

    # -------- METHODS --------

    # Sets graphic object
    def setGraphic(self,graphic):
	if (self.getPalette()==False):
	    return
	self.__graphic=graphic
	self.setCurrentFrameNum(0)

    # Returns graphic object
    def getGraphic(self):
	return self.__graphic
	
    # Sets palette object
    def setPalette(self,palette):
	self.__palette=palette
    
    # Returns palette object
    def getPalette(self):
	return self.__palette
	
    # Sets current frame number
    def setCurrentFrameNum(self,framenum):
	if (self.getGraphic()==False):
	    # Clear framebuffer
	    self.setFrameBuf(False)
	    return
	if (self.getGraphic().countframes()==False):
	    # Clear framebuffer
	    self.setFrameBuf(False)
	    return
	if (framenum>=self.getGraphic().countframes()):
	    return
	self.__curframe=framenum
	self.__createFrameBuf()
	
    # Returns current frame number
    def getCurrentFrameNum(self):
	return self.__curframe
	
    # Returns raw frame object
    def getFrame(self,num):
	return self.getGraphic().getframe(num)
	
    # Sets frame buffer image
    def setFrameBuf(self,imgbuf):
	self.__framebuf=imgbuf

    # Returns frame buffer image
    def getFrameBuf(self):
	return self.__framebuf

    # Sets relative painting position tuple
    def setRelPos(self,x,y):
	self.__relposx=x
	self.__relposy=y

    # Returns relative painting position tuple
    def getRelPos(self):
	return (self.__relposx,self.__relposy)

